@@ -116,12 +116,19 @@ class ReturnValueListener:
         self.return_value = None
 
     def get_last_value(self):
-        if self.return_value is not None and self.return_value != "" and self.return_value != b"":
-            value = to_mime_and_metadata(self.return_value)
+        if not self.has_value(self.return_value):
+            return None
 
-            self.return_value = None
-            return value
-        return None
+        value = to_mime_and_metadata(self.return_value)
+        self.return_value = None
+        return value
+
+    @staticmethod
+    def has_value(value):
+        try:
+            return value is not None and value != "" and value != b""
+        except Exception:
+            return True
 
 
 def clear_drivers(drivers, type_):
@@ -204,19 +211,21 @@ class RpaBrowserConnectionsListener:
         self.drivers = drivers
 
     def end_suite(self, name, attributes):
-        try:
-            instance = BuiltIn().get_library_instance("RPA.Browser")
-            clear_drivers(self.drivers, "RPA.Browser")
-            self.drivers.extend(get_webdrivers(instance._drivers, "RPA.Browser"))
-        except RuntimeError:
-            pass
+        for library in ("RPA.Browser.Selenium", "RPA.Browser"):
+            try:
+                instance = BuiltIn().get_library_instance(library)
+                clear_drivers(self.drivers, library)
+                self.drivers.extend(get_webdrivers(instance._drivers, library))
+            except RuntimeError:
+                pass
 
     def start_suite(self, name, attributes):
-        try:
-            instance = BuiltIn().get_library_instance("RPA.Browser")
-            set_webdrivers(self.drivers, instance._drivers, "RPA.Browser")
-        except RuntimeError:
-            pass
+        for library in ("RPA.Browser.Selenium", "RPA.Browser"):
+            try:
+                instance = BuiltIn().get_library_instance(library)
+                set_webdrivers(self.drivers, instance._drivers, library)
+            except RuntimeError:
+                pass
 
 
 class JupyterConnectionsListener:
